@@ -2,15 +2,16 @@
 
 __docformat__ = 'restructuredtext'
 
-__version__ = '0.4.1'
+__version__ = '0.5'
 
-import models
-import models as model # remain backwards compatibility
 import likelihoods
 import generate
 import utils
+import models
+import cdfdif_wrapper
 
 from models import *
+from kabuki import analyze
 
 import wfpt
 
@@ -19,4 +20,15 @@ try:
 except ImportError:
     pass
 
+
+try:
+    import cdfdif_wrapper as cdfdif
+except ImportError:
+    pass
+
 from kabuki.utils import load_csv, save_csv, load
+
+try:
+    from IPython.core.debugger import Tracer; debug_here = Tracer()
+except:
+    def debug_here(): pass
