@@ -5,6 +5,8 @@ import pymc as pm
 import hddm
 import sys
 import kabuki
+import pandas as pd
+import string
 
 from scipy.stats import scoreatpercentile
 from scipy.stats.mstats import mquantiles
@@ -503,7 +505,7 @@ def _gen_statistics():
 
     return statistics
 
-def gen_ppc_stats(quantiles = (10, 30, 50, 70, 90)):
+def gen_ppc_stats(quantiles=(10, 30, 50, 70, 90)):
     """Generate default statistics for posterior predictive check on
     RT data.
 
@@ -513,21 +515,21 @@ def gen_ppc_stats(quantiles = (10, 30, 50, 70, 90)):
     from collections import OrderedDict
 
     stats = OrderedDict()
-    stats['accuracy'] = lambda x: np.mean(x>0)
+    stats['accuracy'] = lambda x: np.mean(x.rt>0)
 
     #upper bound stats
-    stats['mean_ub'] = lambda x: np.mean(x[x>0])
-    stats['std_ub'] = lambda x: np.std(x[x>0])
+    stats['mean_ub'] = lambda x: np.mean(x.ix[x.rt>0].rt)
+    stats['std_ub'] = lambda x: np.std(x.ix[x.rt>0].rt)
     for q in quantiles:
         key = str(q) + 'q'
-        stats[key+'_ub'] = lambda x, q=q: scoreatpercentile(x[x>0], q) if np.any(x>0) else np.nan
+        stats[key+'_ub'] = lambda x, q=q: scoreatpercentile(x.ix[x.rt>0].rt.values, q) if np.any(x.rt>0) else np.nan
 
     #lower bound stats
-    stats['mean_lb'] = lambda x: np.mean(x[x<0])
-    stats['std_lb'] = lambda x: np.std(x[x<0])
+    stats['mean_lb'] = lambda x: np.mean(x.ix[x.rt<0].rt)
+    stats['std_lb'] = lambda x: np.std(x.ix[x.rt<0].rt)
     for q in quantiles:
         key = str(q) + 'q'
-        stats[key+'_lb'] = lambda x, q=q: scoreatpercentile(np.abs(x[x<0]), q) if np.any(x<0) else np.nan
+        stats[key+'_lb'] = lambda x, q=q: scoreatpercentile(np.abs(x.ix[x.rt<0].rt.values), q) if np.any(x.rt<0) else np.nan
 
     return stats
 
@@ -569,10 +571,33 @@ def plot_posteriors(model, **kwargs):
     pm.Matplot.plot(model.mc, **kwargs)
 
 
-def data_plot(data, nbins=50):
-    data = hddm.utils.flip_errors(data)
-    plt.figure()
-    plt.hist(data['rt'], nbins)
+def data_plot(model, bins=50, nrows=3):
+    nplots = len(model.get_observeds())
+    if nplots < nrows:
+        nrows = nplots
+    ncols = int(np.ceil(nplots / nrows))
+
+    bin_edges = np.linspace(0, np.abs(model.data.rt).max(), bins+1)
+
+
+    fig, axs = plt.subplots(nrows, ncols, sharex=True, sharey=True)
+    if not isinstance(axs, np.ndarray):
+        axs = np.array([axs])
+    axs = axs.flatten()
+    for (i_plt, (name, node_row)) in enumerate(model.iter_observeds()):
+        node = node_row['node']
+        ax = axs[i_plt]
+        for i in range(2):
+            if i == 0:
+                value = node.value[node.value > 0]
+            else:
+                value = np.abs(node.value[node.value < 0])
+
+            counts, bin_edges = np.histogram(value, bins=bin_edges)
+            dx = (bin_edges[1] - bin_edges[0]) / 2.
+            ax.plot(bin_edges[:-1] + dx, counts, lw=2.)
+        ax.set_title(pretty_tag(node_row['tag']))
+
     plt.show()
 
 
@@ -618,21 +643,27 @@ def _plot_posterior_quantiles_node(node, axis, quantiles=(.1, .3, .5, .7, .9),
     axis.set_xlim(value_range)
     axis.set_ylim((0, 1))
 
-    theo = np.empty((2, 2, len(quantiles), samples))
-    for sample in range(samples):
+    sq_lower = np.empty((len(quantiles), samples))
+    sq_upper = sq_lower.copy()
+    sp_upper = np.empty(samples)
+    for i_sample in range(samples):
         kabuki.analyze._parents_to_random_posterior_sample(node)
-        theo[:, :, :, sample] = node.quantiles(quantiles)
+        sample_values = node.random()
+        sq_lower[:, i_sample], sq_upper[:, i_sample], sp_upper[i_sample] = data_quantiles(sample_values)
 
+    y_lower = np.dot(np.atleast_2d(quantiles).T, np.atleast_2d(1 - sp_upper))
+    y_upper = np.dot(np.atleast_2d(quantiles).T, np.atleast_2d(sp_upper))
     if hexbin:
         if predictive_plot_kwargs is None:
             predictive_plot_kwargs = {'gridsize': 75, 'bins': 'log', 'extent': (value_range[0], value_range[1], 0, 1)}
-        axis.hexbin(theo[:,0,:,:].flatten(), theo[:,1,:,:].flatten(), label='post pred lb', **predictive_plot_kwargs)
+        x = np.concatenate((sq_lower, sq_upper))
+        y = np.concatenate((y_lower, y_upper))
+        axis.hexbin(x.flatten(), y.flatten(), label='post pred lb', **predictive_plot_kwargs)
     else:
         if predictive_plot_kwargs is None:
             predictive_plot_kwargs = {'alpha': .75}
-        axis.plot(theo[0,0,:,:], theo[0,1,:,:], label='post pred lb', color='b', **predictive_plot_kwargs)
-        axis.plot(theo[1,0,:,:], theo[1,1,:,:], label='post pred ub', color='r', **predictive_plot_kwargs)
-
+        axis.plot(sq_lower, y_lower, 'o', label='post pred lb', color='b', **predictive_plot_kwargs)
+        axis.plot(sq_upper, y_upper, 'o', label='post pred ub', color='r', **predictive_plot_kwargs)
 
     # Plot data
     data = node.value
@@ -641,9 +672,7 @@ def _plot_posterior_quantiles_node(node, axis, quantiles=(.1, .3, .5, .7, .9),
         data_plot_kwargs = {'color': color, 'lw': 2., 'marker': 'o', 'markersize': 7}
 
     if len(data) != 0:
-        p_upper = np.mean(data>0)
-        q_lower = mquantiles(-data[data<0], quantiles)
-        q_upper = mquantiles(data[data>0], quantiles)
+        q_lower, q_upper, p_upper = data_quantiles(data)
 
         axis.plot(q_lower, quantiles*(1-p_upper), **data_plot_kwargs)
         axis.plot(q_upper, quantiles*p_upper, **data_plot_kwargs)
@@ -652,6 +681,7 @@ def _plot_posterior_quantiles_node(node, axis, quantiles=(.1, .3, .5, .7, .9),
     axis.set_ylabel('Prob respond')
     axis.set_ylim(bottom=0) # Likelihood and histogram can only be positive
 
+
 def plot_posterior_quantiles(model, **kwargs):
     """Plot posterior predictive quantiles.
 
@@ -693,11 +723,10 @@ def plot_posterior_quantiles(model, **kwargs):
 
     if 'value_range' not in kwargs:
         rt = np.abs(model.data['rt'])
-        kwargs['value_range'] = (np.min(rt.min()-.2, 0), rt.max()-.2)
+        kwargs['value_range'] = (np.min(rt.min()-.2, 0), rt.max())
 
     kabuki.analyze.plot_posterior_predictive(model,
                                              plot_func=_plot_posterior_quantiles_node,
-                                             required_method='quantiles',
                                              **kwargs)
 
 
@@ -708,6 +737,103 @@ def create_test_model(samples=5000, burn=1000, subjs=1, size=100):
 
     return m
 
+def pretty_tag(tag):
+    return tag[0] if len(tag) == 1 else string.join(tag, ', ')
+
+def qp_plot(model, quantiles=(0.1, 0.3, 0.5, 0.7, 0.9), ncols=None):
+    """
+    qp plot
+    Input:
+        model : HDDM model
+
+        quantiles : sequence
+            sequence of quantiles
+
+        ncols : int
+            number of columns in output figure
+    """
+
+    #if model is a group model then we create an average model and plot it
+    if model.is_group_model:
+        avg_model = model.get_average_model()
+        qp_plot(avg_model, quantiles=quantiles)
+
+    #create axes
+    n_subjs = model.num_subjs
+    if ncols is None:
+        ncols = min(4, n_subjs)
+    nrows = int(np.ceil(n_subjs / ncols))
+    fig, axs = plt.subplots(nrows, ncols, sharex=True, sharey=False)
+
+    #plot single subject model
+    obs = model.get_observeds()
+    if not model.is_group_model:
+        axs = np.array([axs])
+        _qp_plot_of_nodes_db(obs, quantiles=quantiles, ax=axs[0])
+
+    #plot group model
+    else:
+        for i, (subj_idx, t_nodes_db) in enumerate(obs.groupby('subj_idx')):
+            _qp_plot_of_nodes_db(t_nodes_db, quantiles=quantiles, ax=axs.flat[i])
+
+    #add legend
+    leg = axs.flat[0].legend(loc='best', fancybox=True)
+    if leg is not None:
+        leg.get_frame().set_alpha(0.5)
+
+def _qp_plot_of_nodes_db(nodes_db, quantiles, ax):
+    """
+    function used by qp_plot
+    Input
+        nodes_db : DataFrame
+            DataFram of observeds nodes (e.g. model.get_observeds())
+
+        quantiles : sequence
+            sequence of quantiles
+
+        ax : axes
+            axes to plot on
+    """
+
+    # ax.set_xlim(0,1)
+    nq = len(quantiles)
+    nodes_db = nodes_db.sort_index()
+
+    #loop over nodes
+    for name, node_row in nodes_db.iterrows():
+
+        #get quantiles
+        q_lower, q_upper, p_upper = node_row['node'].empirical_quantiles(quantiles)
+
+        #plot two lines for each node
+        tag = node_row['tag']
+        tag = pretty_tag(tag)
+        line = ax.plot(np.ones(nq)*p_upper, q_upper, '-x', label=tag)[0]
+        ax.plot(np.ones(nq)*(1-p_upper), q_lower, '-x', c=line.get_color())[0]
+
+
+def data_quantiles(data, quantiles=(0.1, 0.3, 0.5, 0.7, 0.9)):
+    """
+    compute the quantiles of 2AFC data
+
+    Output:
+        q_lower - lower boundary quantiles
+        q_upper - upper_boundary_quantiles
+        p_upper - probability of hitting the upper boundary
+    """
+    if isinstance(data, pd.DataFrame):
+        if 'response' in data.columns:
+            data = flip_errors(data).rt.values
+        else:
+            data = data.rt.values
+
+    quantiles = np.asarray(quantiles)
+    p_upper = float(np.mean(data>0))
+    q_lower = mquantiles(-data[data<0], quantiles)
+    q_upper = mquantiles(data[data>0], quantiles)
+
+    return q_lower, q_upper, p_upper
+
 if __name__ == "__main__":
     import doctest
     doctest.testmod()
